#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Helper module.

Main entrypoint for processing template operations.

"""

import os
from jinja2 import TemplateNotFound, Environment, FileSystemLoader

from pyth.utils import find_template_dir, sanitize_path


class Helper(): # pylint: disable=too-few-public-methods
    """Main helper class.

    This class is the glue for the template and data. The result is outputed
    in a file in the current directory.

    :param templates_dir: The path of templates directory intended to
        override the default.
    :type templates_dir: str
    """

    __template_dir = None
    __loader = None
    __environment = None

    def __init__(self, templates_dir=None):
        self.__init_template_context(templates_dir)

    def __init_template_context(self, path):
        """Initialisation method.

        Prepare the template context of the helper. It is based on the
        file system loader of Jinja2 librairies.
        """

        _override_path = None
        if path:
            path = sanitize_path("", basedir=path)
            if os.path.exists(path):
                _override_path = path
        self.__template_dir = _override_path or find_template_dir()

        if self.__template_dir:
            self.__loader = FileSystemLoader(self.__template_dir)
            self.__environment = Environment(loader=self.__loader)

    def render_template(self, template_name, context_data=None):
        """Process template with input data.

        Renders the template accoding to data with a simple
        jinja render template method.

        :param template_name: The name of the template to be processed.
        :param context_data: The data to pass to the template. It can be an
            iterable or dict.
        :type template_name: str
        :type context_data: object
        :rtype: str
        :returns: The rendered text from template and data.
        """

        if not self.__template_dir:
            print("Warning - Template directory not set.")
            return None

        try:
            _template = self.__environment.get_template(template_name)
            _rendered_text = _template.render(data=context_data)
            return _rendered_text
        except TemplateNotFound as tnf:
            print(f"Error with template - '{tnf}'.")
            print(f"Not found in '{self.__template_dir}'.")
            print("Make sure template exists and corretcly named.")
            return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Utils functions.

This module is a groups for extra utils functions used
in other packages or modules.
"""

import os
import csv


def sanitize_path(path, basedir=None):
    """Sanitize file path.

    Clean the string path of file. Expands environment variables,
    relative paths and symbols. `Note` : It does not check whether
    the path exists.

    :param path: The string representing a path to be sanitized.
    :type path: str
    :rtype: str
    :returns: An absolute path of the file.
    """

    if basedir is None:
        basedir = os.getcwd()
    elif os.path.isfile(basedir):
        basedir = os.path.dirname(basedir)

    final_path = os.path.expanduser(os.path.expandvars(path))
    final_path = os.path.join(basedir, final_path)

    return os.path.normpath(final_path)

def load_csv(path, delimiter=','):
    """Load csv data from file.

    This function read csv file and uses first line as header and returns
    a list of dict object.

    :param path: The string representing a path of csv file.
    :type path: str
    :rtype: list
    :returns: List of dict object.
    :raises FileNotFoundError: if the path of file does not exist.
    """

    csv_path = sanitize_path(path)
    if not os.path.exists(csv_path):
        raise FileNotFoundError(f"File not found: '{csv_path}'")

    # load data
    with open(csv_path, 'rt') as csvfile:
        reader = csv.DictReader(csvfile, delimiter=delimiter)
        data = [dict(row) for row in reader]

    return data

def find_template_dir():
    """Template dir finder.

    Search for a directory named ``templates``. The first found is used.
    Loading order:

    * environment variable: $PYTH_CONFIG_DIR/templates
    * working directory: $CWD/templates
    * home directory: $HOME/.pyth/templates
    * system default: /etc/pyth/templates

    :rtype: str
    :return: The first path found for the template base dir.
    """

    potential_paths = []

    # Environment path
    path_from_env = os.getenv("PYTH_CONFIG_DIR")
    if path_from_env:
        path_from_env = sanitize_path(path_from_env)
        potential_paths.append(os.path.join(path_from_env, "templates"))

    # Templates in current working directory
    path_from_cwd = os.path.join(os.getcwd(), "templates")
    potential_paths.append(path_from_cwd)

    # Home user directory
    potential_paths.append(sanitize_path("~/.pyth/templates"))

    # System default templates
    potential_paths.append("/etc/pyth/templates")

    for path in potential_paths:
        if os.path.exists(path):
            break
    else:
        path = None

    return path



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
from setuptools import setup


if sys.version_info.major < 3:
    raise RuntimeError(
            "Pyth does not support Python 2.x."
            "Please use Python 3.")

setup()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Pyth entrypoint.
"""

import os
import sys
import argparse

from pyth.helper import Helper
from pyth.utils import load_csv, sanitize_path
from pyth import version


def main(argv=None):
    """The ``pyth`` program can be used as CLI with main function entrypoint.
    """

    # init arg parser
    parser = argparse.ArgumentParser(
        prog="pyth", description="Outputs the result of a Jinja2 template.")
    # positional arguments
    parser.add_argument("template", help="The name of template.")
    parser.add_argument("csv", help="The path to csv file.")

    # optional arguments
    parser.add_argument("--version", action="version",
                        version="%%(prog)s version: %s" % version.__version__,
                        help="Print program's version number and exit.")
    parser.add_argument("-o", "--output", metavar="FILE",
                        help="The path of output file.")
    parser.add_argument("-t", "--templates-dir", metavar="DIRECTORY",
                        help="The path of directory containing templates.")
    # parse args
    args = parser.parse_args(argv)

    # get csv data
    data = None
    try:
        data = load_csv(args.csv)
    except FileNotFoundError as fnf:
        print(f"Error with csv file - {fnf}")
        print("Attempt to render template with no data.")

    helper = Helper(templates_dir=args.templates_dir)
    result = helper.render_template(args.template, context_data=data)

    if result:
        # write output
#        cwd = os.getcwd()
#        output_file = os.path.join(cwd, 'output.txt')
        if args.output:
            output_file = sanitize_path(args.output)
            # file mode
            mode = 'w' if os.path.exists(output_file) else 'a'
            with open(output_file, mode) as file:
                file.write(result)
        else:
            print(result)
        sys.exit(0)
    else:
        sys.exit("Nothing to write in file.")

if __name__ == '__main__': # pragma: no cover
    main()
